' ******************************************************************************
' *
' *  OSVERCMD - geschrieben in / written in FreeBASIC
' *
' *  Gibt die Betriebssystemversion auf der Kommandozeile zurck /
' *  Returns information about the os version and build
' *
' *  VERSION: 1.1, 20.05.2014, Tobias Jacobi
' *
' *
' *  HAFTUNGSAUSSCHLUSS
' *
' *  Dieser Quellcode wird kostenlos fr jegliche Verwendung zur Verfgung
' *  gestellt, aber ohne irgendeine Garantie oder Haftung. Sie knnen ihn
' *  auf eigene Gefahr verwenden oder verndern.
' *
' *  DISCLAIMER
' *
' *  This source code is provided free for all purposes, but without any
' *  warranty. Use or change it at your own risk.
' *
' *
' *  Ntzliche Informationen und Software zu FreeBasic finden Sie hier /
' *  You can find some useful informations and software around FreeBasic here:
' *
' *  http://www.freebasic.net/
' *  http://www.freebasic-portal.de/
' *
' ******************************************************************************

' Notwendige Sourcen einbinden
#Include "windows.bi"

' Funktionen und Prozeduren deklarieren
Declare Function MyRegRead(ByVal P_RootKey As HKEY, ByVal P_KeyPath As LPCSTR, _
                           ByVal P_ValueName As LPCSTR) As String
Declare Function MyGetArchitecture() As String

' Globale Variablen und Konstanten deklarieren und ggf. initialisieren
Dim As OSVERSIONINFO GV_OSV                     
Dim As Integer       GV_Result, GV_UnknownArg
Dim As String        GV_ResultString, GV_CurArg
Dim As String        GV_FBErrorCodes(0 To 17) => {_
                       "No error", _
                       "Illegal function call", _
                       "File not found", _
                       "File I/O error", _
                       "Out of memory", _
                       "Illegal resume", _ 
                       "Out of bounds array access", _ 
                       "Null pointer access", _
                       "No privileges", _
                       "Interrupted signal", _
                       "Illegal instruction signal", _ 
                       "Floating point error signal", _
                       "Segmentation violation signal", _
                       "Termination request signal", _
                       "Abnormal termination signal", _
                       "Quit request signal", _
                       "Return without gosub", _
                       "End of file"}

' Fehlerbehandlung einschalten
On Error GoTo GLABEL_Error

' Parameter auswerten (es wird nur der erste ausgewertet)
GV_UnknownArg = 0
GV_CurArg = Trim(Command(1))
If GV_CurArg <> "" Then
  ' Ein Argument muss 2-stellig sein und mit - oder / beginnen
  If Len(GV_CurArg) = 2 And _
     (Left(GV_CurArg, 1) = "/" Or Left(GV_CurArg, 1) = "-") Then
    ' Die nchste Stelle im Argument fr die sptere Rckgabe merken
    Select Case UCase(Mid(GV_CurArg, 2, 1))
      Case "O"
        GV_ResultString = "OS_ONLY"
      Case "B"
        GV_ResultString = "BUILD_ONLY"
      Case "P"
        GV_ResultString = "PRODUCTTYPE_ONLY"
      Case "A"
        GV_ResultString = "ARCHITECTURE_ONLY"
      Case Else
        GV_UnknownArg = 1
    End Select
  Else
    GV_UnknownArg = 1
  End If
End If

' Bei unbekannten Parametern Hilfetext ausgeben und beenden
If GV_UnknownArg <> 0 Then
  Print "OSVERCMD 1.1 - written 2011/2014 in FreeBASIC by Tobias Jacobi"
  Print ""
  Print "Prints information about the os version and build. The output can be"
  Print "formatted for usage in batch files:"
  Print ""
  Print "OSVERCMD.EXE [/O | /B | /P | /A]"
  Print ""
  Print "/O  Returns the os version only (PlatformID.MajorVersion.MinorVersion)"
  Print "/B  Returns the build number only"
  Print "/P  Returns the product type only"
  Print "/A  Returns the architecture only (64-bit / 32-bit)"
  Print ""
  Print "Example batch file:"
  Print ""
  Print "@ECHO off"
  Print "FOR /f %%i in ('osvercmd.exe /o') do set MYOSVERS=%%i"
  Print "ECHO %MYOSVERS%"
  Print ""
  Print "Look at the Microsoft Knowledge Base for the returned os versions."
  Print "Windows XP returns 2.5.1 for example. Currently known product types"
  Print "are WinNT for clients and LanmanNT or ServerNT for servers."   
  Print "This program is free of charge for private and commercial use, but"
  Print "without any warranty. You use it at your own risk."
  End
End If

' Versionsinfo einholen
GV_OSV.dwOSVersionInfoSize = SizeOf(GV_OSV)
GV_Result                  = GetVersionEx(@GV_OSV)

' Bei erfolgreicher Abfrage die gewnschte Ausgabe generieren
IF GV_Result <> 0 Then
  If GV_ResultString = "BUILD_ONLY" Then
    ' Nur die Build-Nummer zurckgeben
    Print Str(GV_OSV.dwBuildNumber)
  ElseIf GV_ResultString = "OS_ONLY" Then
    ' Nur die OS Version (PlatformID.MajorVersion.MinorVersion) zurckgeben
    Print Str(GV_OSV.dwPlatformId) & "." & Str(GV_OSV.dwMajorVersion) & "." & _
          Str(GV_OSV.dwMinorVersion)
  ElseIf GV_ResultString = "PRODUCTTYPE_ONLY" Then
    ' Nur den Product Type zurckgeben
    Print MyRegRead(HKEY_LOCAL_MACHINE, _
                    "SYSTEM\CurrentControlSet\Control\ProductOptions", _
                    "ProductType")
  ElseIf GV_ResultString = "ARCHITECTURE_ONLY" Then
    ' Nur die Architektur zurckgeben
    Print MyGetArchitecture
  Else
    ' Sonst eine Liste mit allen Infos ausgeben
    Print "Platform ID ......: " & Str(GV_OSV.dwPlatformId)
    Print "Major Version ....: " & Str(GV_OSV.dwMajorVersion)
    Print "Minor Version ....: " & Str(GV_OSV.dwMinorVersion)
    Print "Build ............: " & Str(GV_OSV.dwBuildNumber)
    Print "Product Type .....: " & MyRegRead(HKEY_LOCAL_MACHINE, _
                                  "SYSTEM\CurrentControlSet\Control\" & _
                                  "ProductOptions", "ProductType")
    Print "Architecture .....: " & MyGetArchitecture
  End If
Else
  Print "ERROR: Function GetVersionEx failed!!!"
End If
End

' Behandelt aufgetretene Fehler 
GLABEL_Error:
GV_Result = Err
GV_ResultString = "ERROR: #" & Str(Err) & " at line #" & Str(Erl)
If GV_Result >= 0 And GV_Result <= 17 Then
  GV_ResultString &= "; " & GV_FBErrorCodes(GV_Result)
End If
Print GV_ResultString
Print ""
Print "You can find a complete list of runtime error codes in the"
Print "FreeBASIC documentation."
End


' ******************************************************************************
' *  Funktionen und Prozeduren
' ******************************************************************************
Function MyRegRead(ByVal P_RootKey As HKEY, ByVal P_KeyPath As LPCSTR, _
                   ByVal P_ValueName As LPCSTR) As String
  ' Gibt den Wert mit Namen ValueName aus der Registry unterhalb von
  ' RootKey -> KeyPath als String zurck, im Fehlerfall wird ein
  ' String beginnend mit ERROR_MyRegRead_ zurckgegeben
  Const                        LC_MaxValueLen = 2048
  Dim As String*LC_MaxValueLen LV_ValueData = ""
  Dim As String                LV_ValueDataBinary = ""
  Dim As DWORD                 LV_ValueDataType, LV_ValueDataDword, _
                               LV_ValueLen, LV_Counter
  Dim As HKEY                  LV_KeyHandle
  Dim As Integer               LV_Result

  ' Handle zum Key ffnen
  LV_Result    = RegOpenKeyEx(P_RootKey, P_KeyPath, 0, KEY_READ, @LV_KeyHandle)
  ' Prfen, ob der Handle geffnet werden konnte
  If LV_Result = ERROR_SUCCESS Then
    ' Falls ja, dann angegebenen Wert abfragen
    LV_ValueLen  = LC_MaxValueLen
    LV_Result = RegQueryValueEx(LV_KeyHandle, P_ValueName, _
                                NULL, @LV_ValueDataType, _
                                Cast(Byte Ptr, @LV_ValueData), _
                                @LV_ValueLen)
    ' Prfen, ob der Wert abgefragt werden konnte
    If LV_Result = ERROR_SUCCESS Then
      ' Art des Rckgabewerts von RegQueryValueEx entsprechend behandeln 
      Select Case LV_ValueDataType
        Case REG_DWORD
          ' Bei REG_DWORD String in ein echtes DWORD kopieren
          CopyMemory(@LV_ValueDataDword, @LV_ValueData, 4)
          ' Dann wieder in einen sauberen String umwandeln
          LV_ValueData = Str(LV_ValueDataDword)
        Case REG_BINARY
          ' Bei REG_BINARY einen String mit Hex-Werten erzeugen
          For LV_Counter = 1 To LV_ValueLen
            LV_ValueDataBinary &= Hex(Asc(Mid(LV_ValueData, LV_Counter, 1)), 2) 
          Next
          ' Den Binary-Wert auf MaxValueLen krzen und zurckliefern
          LV_ValueData = Left(LV_ValueDataBinary, LC_MaxValueLen) 
        Case Else
          ' Es ist schon ein String, dann in passender Lnge zurckliefern
          LV_ValueData = Left(LV_ValueData, LV_ValueLen - 1)
      End Select
    Else
      ' Sonst Fehlerwert zurckliefern
      LV_ValueData = "ERROR_MyRegRead_RegQueryValueEx!!!"
    End If
    ' Den geffneten Handle wieder schlieen
    LV_Result = RegCloseKey(LV_KeyHandle)
  Else
    ' Sonst Fehlerwert zurckliefern
    LV_ValueData = "ERROR_MyRegRead_RegOpenKeyEx!!!"
  End If
  ' Funktionsergebnis setzen
  Return LV_ValueData
End Function

Function MyGetArchitecture() As String
  ' Gibt 64-bit oder 32-bit entsprechend der Windows-Variante zurck 
  Dim As String LV_Architecture, LV_Buffer
  ' Registry auslesen
  LV_Buffer = MyRegRead(HKEY_LOCAL_MACHINE, _
                        "HARDWARE\DESCRIPTION\System\CentralProcessor\0", _
                        "Identifier")
  ' Wurde ein brauchbarer String gefunden?
  If Len(LV_Buffer) > 3 Then
    ' Wenn der String nicht mit "x86" beginnt und die Umgebungsvariable
    ' "ProgramFiles(x86)" existiert, dann ist es auch 64-bit 
    If UCase(Left(LV_Buffer, 3)) <> "X86" And _
       Len(Trim(Environ("ProgramFiles(x86)"))) > 0 Then
      LV_Architecture = "64-bit"
    Else
      LV_Architecture = "32-bit"
    EndIf
  Else
    LV_Architecture = "ERROR"
  EndIf
  ' Funktionsergebnis setzen
  Return LV_Architecture
End Function
